/*
 *
 *  JUMA-TRX2 External Keyboard Ver 1.0
 *
 * This code is for Microchip 16F628 controller and is compiled
 * with Hitec PICC_8.05PL2 compiler.
 *
 * Device low power PIC16F628A
 *
 * OH2NLT / Juha Niinikoski 13.03.2007
 *
 * Switch debounce filters added 13.03.2007
 * Switch layout changed, shift button added 15.01.2008, version changed to v1.01
 *
*/

/* debug controls */
//#define		debug_on		// debug printouts on

#ifdef debug_on
#warning *
#warning *  Debug Option is on   *
#warning *
#endif

/* included headers */
#include <pic.h>
#include "juma-ext_kb.h"

/* global variables */

// timer
bit tick;								// 1 ms event timing flag

// Switch debounce filters
#define NUMCH 10						// number of channells
unsigned char rawinput[NUMCH];			// raw input 0 or 1
unsigned char inputfilter[NUMCH];		// filter shift register
unsigned char filteredinput[NUMCH];		// filtered input 0 or 1
// Switch push down events
unsigned char inputchanged_0[NUMCH];	// set to 1 if changed to 0, user must reset it
// Switch release events
unsigned char inputchanged_1[NUMCH];	// set to 1 if changed to 1, user must reset it

// Command characters SW1 ... SW10 V1.00 button order
//const char cmd[NUMCH] = {'0', '1', '2', '3', 'T', 'P', 'M', 'R', '4', 'S'};

// Command characters SW1 ... SW10 V1.01 buttonorder
const char cmd[NUMCH] = {'M', 'R', 'P', 'T', '4', '3', '2', '1', 'S', '0'};
const char s_cmd[NUMCH] = {'M', 'R', 'P', 'T', '9', '8', '7', '6', 'S', '5'}; // shift button down

/* included programs */

#include "timer.c"
#include "miniio.c"

/* processor config & id info */

	__CONFIG(UNPROTECT & XT & WDTDIS & PWRTEN & BOREN & LVPDIS);	// 

	__IDLOC(0801);									// Device ID

const char ver[] = {"1.0.1"};												// Version
const char id[] = {"\n\nJUMA-ext_kb OH2NLT Juha Niinikoski 15.01.2008\n"}; 	// id string


/* helper routines */

// Check switch events & send commands
void send_cmd(void)
	{
	char x;
	for(x=0; x<NUMCH; x++)							// do all channells
		{
		if(inputchanged_0[x] == 1)					// push down event ?
			{
			inputchanged_0[x] = 0;					// reset event
			if(SW11 == 0)							// shift button pressed
				putch(s_cmd[x]);					// send command from shift table
			else
				putch(cmd[x]);						// send command from normal table
			}
		}
	}


// collect & format switch bits
// this routine is very hardware dependent
// input states are stored in rawinput[] array

void collect_input(void)
	{
	if(SW1 == 1)				// resolve SW I/O bit by bit
		rawinput[0] = 1;		// not fancy but easy way
	else
		rawinput[0] = 0;

	if(SW2 == 1)
		rawinput[1] = 1;
	else
		rawinput[1] = 0;

	if(SW3 == 1)
		rawinput[2] = 1;
	else
		rawinput[2] = 0;

	if(SW4 == 1)
		rawinput[3] = 1;
	else
		rawinput[3] = 0;

	if(SW5 == 1)
		rawinput[4] = 1;
	else
		rawinput[4] = 0;

	if(SW6 == 1)
		rawinput[5] = 1;
	else
		rawinput[5] = 0;

	if(SW7 == 1)
		rawinput[6] = 1;
	else
		rawinput[6] = 0;

	if(SW8 == 1)
		rawinput[7] = 1;
	else
		rawinput[7] = 0;

	if(SW9 == 1)
		rawinput[8] = 1;
	else
		rawinput[8] = 0;

	if(SW10 == 1)
		rawinput[9] = 1;
	else
		rawinput[9] = 0;
	}


// input debounce filter, handle also N-key rollover

void run_input_filter(void)
	{
	char x;

	for(x=0; x<NUMCH; x++)								// do all channells
		{
		inputfilter[x] = inputfilter[x] >> 1;			// fill filters at MSB

		if(rawinput[x] == 0)
			inputfilter[x] = inputfilter[x] & 0x7F;		// drop MSB
		else
			inputfilter[x] = inputfilter[x] | 0x80;		// set MSB

		if(inputfilter[x] == 0xFF)						// we have got 1
			{
			if(filteredinput[x] == 0)					// compare with old value
				inputchanged_1[x] = 1;					// set flag if changed to 1
			filteredinput[x] = 1;						// store new value
			}

		if(inputfilter[x] == 0x00)						// we have got 0
			{
			if(filteredinput[x] == 1)					// compare with old value
				inputchanged_0[x] = 1;					// set flag if changed to 0
			filteredinput[x] = 0;						// store new value
			}		
		}	
	}



/* Main control loop */

void main(void)
{
char x;

	PORTB = PORTB_INIT;				// init ports
	PORTA = PORTA_INIT;	

	CMCON = CMCON_INIT;				// comparators off, all bits for I/O

	TRISA = TRISA_INIT;
#ifdef debug_on
	TRISB = TRISB_DEBUG;
#else
	TRISB = TRISB_INIT;
#endif

	init_timers();					// start timing functions
	serial_setup();					// start serial IO


#ifdef debug_on
	putst(id);						// say hello
#endif


	for(;;)							// run logig
		{

		if(tick)					// run input filters at xx ms rate
			{
#ifdef debug_on
			TEST = 1;				// for timing measurement
#endif
			tick = 0;
			collect_input();		// collect switch bits from I/O
			run_input_filter();		// run input filters
#ifdef debug_on
			TEST = 0;
#endif
			}	

		send_cmd();					// check switch push events & send commands


#if 0
// downstream command interface
		if (keypressed() == 1)		// check if host command
			{
			switch (getch())
				{
				case 'I':			// Print ID string
					putst(id);
				break;

				default:
					putch('?');
				break;
				}
			}
#endif

			
		} 				// end for
}						// end main
